/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "environment.h"

// This constructor initializes all of the widgets and sets their properties. It
// then creates and shows the dialog. It receives the environment settings object
EnvironmentDialog::EnvironmentDialog (EnvironmentSettings *env) 
{
  fontname = Glib::ustring(env->get_font_face()) + " " + Glib::ustring(env->get_font_size());
  
  // Create objects for dialog
  initialize_objects();
  
  set_default_icon_from_file(string(INSTALL_PREFIX) + "/share/pixmaps/openldev/openldev24.png");
  get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  ok->set_flags(Gtk::CAN_DEFAULT);
  
  // Create the General editor tab
  label_tabstops->set_alignment(0,0.5);
  label_autoindent->set_alignment(0,0.5);
  label_filemanagement->set_alignment(0,0.5);
  label_undo->set_alignment(0,0.5);
  label_tabwidth->set_padding(10,0);
  label_autosave->set_padding(10,0);
  label_minutes->set_padding(10,0);
  label_limitto->set_padding(10,0);
  label_actions->set_padding(10,0);
  tab_size->set_digits(0);
  tab_size->set_value(env->get_tabsize());
  asave_minutes->set_digits(0);
  asave_minutes->set_value(env->get_autosave());
  undo_actions->set_digits(0);
  undo_actions->set_value(env->get_undo());
  replace_tabs->set_active(env->get_replace_tabs());
  load_on_launch->set_active(env->get_autoload());
  autoindent->set_active(env->get_auto_indent());
  hbox_tabsize->pack_start(*label_tabwidth, Gtk::PACK_SHRINK, 0);
  hbox_tabsize->pack_start(*tab_size, Gtk::PACK_SHRINK, 0);
  hbox_autosave->pack_start(*label_autosave, Gtk::PACK_SHRINK, 0);
  hbox_autosave->pack_start(*asave_minutes, Gtk::PACK_SHRINK, 0);
  hbox_autosave->pack_start(*label_minutes, Gtk::PACK_SHRINK, 0);
  fixed_tabs->put(*replace_tabs, 16, 0);
  fixed_loadlaunch->put(*load_on_launch, 16, 0);
  fixed_autoindent->put(*autoindent, 16, 0);
  hbox_undo->pack_start(*label_limitto, Gtk::PACK_SHRINK, 0);
  hbox_undo->pack_start(*undo_actions, Gtk::PACK_SHRINK, 0);
  hbox_undo->pack_start(*label_actions, Gtk::PACK_SHRINK, 0);
  vbox_editor->set_border_width(10);
  vbox_editor->set_spacing(10);
  vbox_editor->pack_start(*label_tabstops, Gtk::PACK_SHRINK, 0);
  vbox_editor->pack_start(*hbox_tabsize, Gtk::PACK_SHRINK, 0);
  vbox_editor->pack_start(*fixed_tabs, Gtk::PACK_SHRINK, 0);
  vbox_editor->pack_start(*label_autoindent, Gtk::PACK_SHRINK, 0);
  vbox_editor->pack_start(*fixed_autoindent, Gtk::PACK_SHRINK, 0);
  vbox_editor->pack_start(*label_filemanagement, Gtk::PACK_SHRINK, 0);
  vbox_editor->pack_start(*hbox_autosave, Gtk::PACK_SHRINK, 0);
  vbox_editor->pack_start(*fixed_loadlaunch, Gtk::PACK_SHRINK, 0);
  vbox_editor->pack_start(*label_undo, Gtk::PACK_SHRINK, 0);
  vbox_editor->pack_start(*hbox_undo, Gtk::PACK_SHRINK, 0);
  
  // Create the Fonts & Colors tab
  label_font->set_alignment(0,0.5);
  label_compiler->set_alignment(0,0.5);
  label_error->set_padding(10,0);
  label_warning->set_padding(10,0);
  label_editor_font->set_padding(10,0);
  error->set_color(Gdk::Color(env->get_error_color()));
  warning->set_color(Gdk::Color(env->get_warning_color()));
  fontbutton->set_font_name(Glib::ustring(env->get_font_face() + " " + env->get_font_size()));
  hbox_font->pack_start(*label_editor_font, Gtk::PACK_SHRINK, 0);
  hbox_font->pack_start(*fontbutton, Gtk::PACK_SHRINK, 0);
  table_ew_color->attach(*label_error, 0, 1, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 0, 5);
  table_ew_color->attach(*label_warning, 0, 1, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 0, 5);
  table_ew_color->attach(*error, 1, 2, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 0, 5);
  table_ew_color->attach(*warning, 1, 2, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 0, 5);
  vbox_font->set_border_width(10);
  vbox_font->set_spacing(10);
  vbox_font->pack_start(*label_font, Gtk::PACK_SHRINK, 0);
  vbox_font->pack_start(*hbox_font, Gtk::PACK_SHRINK, 0);
  vbox_font->pack_start(*label_compiler, Gtk::PACK_SHRINK, 0);
  vbox_font->pack_start(*table_ew_color, Gtk::PACK_SHRINK, 0);

  // Append all of the tabs
  notebook->set_scrollable(true);
  notebook->append_page(*vbox_editor, *label_editor);
  notebook->append_page(*vbox_font, *label_fonts);

  // Create the dialog & show it
  get_vbox()->set_homogeneous(false);
  get_vbox()->set_spacing(0);
  get_vbox()->pack_start(*notebook);
  set_title("Environment Settings");
  set_modal(false);
  set_size_request(325, -1);
  property_window_position().set_value(Gtk::WIN_POS_NONE);
  set_resizable(true);
  property_destroy_with_parent().set_value(false);
  set_has_separator(true);
  add_action_widget(*cancel, -6);
  add_action_widget(*ok, -5);
  show_all_children();
  show();
}

// Initialize all of the objects and set all of the labels to be bold
void EnvironmentDialog::initialize_objects()
{
  Pango::FontDescription fd("Sans Bold 10");
  
  notebook = new Gtk::Notebook();
  fontbutton = new Gtk::FontButton();
  cancel = new Gtk::Button(Gtk::StockID("gtk-cancel"));
  ok = new Gtk::Button(Gtk::StockID("gtk-ok"));
  
  // Initialize all of the labels
  label_tabstops = new Gtk::Label("Tab Stops");
  label_autoindent = new Gtk::Label("Auto Indentation");
  label_tabwidth = new Gtk::Label("  Tab Width:");
  label_filemanagement = new Gtk::Label("File Management");
  label_autosave = new Gtk::Label("  Autosave every");
  label_minutes = new Gtk::Label("minutes");
  label_editor = new Gtk::Label("Editor");
  label_font = new Gtk::Label("Font");
  label_editor_font = new Gtk::Label("  Editor Font:");
  label_compiler = new Gtk::Label("Compiler");
  label_error = new Gtk::Label("  Error Color:     ");
  label_fonts = new Gtk::Label("Fonts & Colors");
  label_vars = new Gtk::Label("  Variables:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label_warning = new Gtk::Label("  Warning Color:");
  label_limitto = new Gtk::Label("  Limit to");
  label_actions = new Gtk::Label("actions");
  label_undo = new Gtk::Label("Undo");

  tab_size_adj = new Gtk::Adjustment(1, 0, 9, 1, 10, 10);
  asave_minutes_adj = new Gtk::Adjustment(1, 0, 99, 1, 10, 10);
  undo_adj = new Gtk::Adjustment(1, 0, 99, 1, 10, 10);
  tab_size = new Gtk::SpinButton(*tab_size_adj, 1, 0);
  asave_minutes = new Gtk::SpinButton(*asave_minutes_adj, 1, 0);
  undo_actions = new Gtk::SpinButton(*undo_adj, 1, 0);
  
  replace_tabs = new Gtk::CheckButton("Insert spaces instead of tabs");
  load_on_launch = new Gtk::CheckButton("Load last project on launch");
  autoindent = new Gtk::CheckButton("Enable automatic indentation");
  
  // Initialize all of the containers
  hbox_tabsize = new Gtk::HBox(false, 0);
  hbox_autosave = new Gtk::HBox(false, 0);
  hbox_font = new Gtk::HBox(false, 0);
  hbox_undo = new Gtk::HBox(false, 0);
  vbox_editor = new Gtk::VBox(false, 7);
  vbox_font = new Gtk::VBox(false, 7);
  fixed_tabs = new Gtk::Fixed();
  fixed_loadlaunch = new Gtk::Fixed();
  fixed_autoindent = new Gtk::Fixed();
  table_ew_color = new Gtk::Table(2, 3, false);
  
  // Initialize all of the color buttons
  error = new Gtk::ColorButton();
  warning = new Gtk::ColorButton();
  
  // Set the labels to have a bold font
  label_tabstops->modify_font(fd);
  label_autoindent->modify_font(fd);
  label_filemanagement->modify_font(fd);
  label_font->modify_font(fd);
  label_compiler->modify_font(fd);
  label_undo->modify_font(fd);
}

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
EnvironmentDialog::~EnvironmentDialog()
{
  delete notebook;
  delete cancel;
  delete ok;
  delete label_tabstops;
  delete label_tabwidth;
  delete label_filemanagement;
  delete label_autosave;
  delete label_minutes;
  delete label_editor;
  delete label_font;
  delete label_editor_font;
  delete label_compiler;
  delete label_error;
  delete label_fonts;
  delete label_warning;
  delete label_autoindent;
  delete label_undo;
  delete label_limitto;
  delete label_actions;
  delete tab_size_adj;
  delete asave_minutes_adj;
  delete undo_adj;
  delete tab_size;
  delete asave_minutes;
  delete undo_actions;
  delete replace_tabs;
  delete load_on_launch;
  delete autoindent;
  delete hbox_tabsize;
  delete hbox_autosave;
  delete hbox_font;
  delete hbox_undo;
  delete vbox_editor;
  delete vbox_font;
  delete fixed_tabs;
  delete fixed_loadlaunch;
  delete fixed_autoindent;
  delete error;
  delete warning;
  delete table_ew_color;
  delete fontbutton;
}

// Get the font size from the FontButton
Glib::ustring EnvironmentDialog::get_font_size()
{ 
  Glib::ustring font = fontbutton->get_font_name();
  
  while (font.find(" ", 0) != string::npos)
    font.erase(0, font.find(" ", 0)+1);
  
  return font;
}

// Get the font face from the FontButton
Glib::ustring EnvironmentDialog::get_font_face()
{ 
  Glib::ustring font = fontbutton->get_font_name();
  
  int loc = font.find(' ', 0);
  while (font.find(' ', loc+1) != string::npos)
    loc = font.find(' ', loc+1);
  
  font.erase(loc, font.length()-loc);
  return font;
}

// These functions allow the openldev class to get private members of this dialog
int EnvironmentDialog::get_asave_minutes()
{ return static_cast<int>(asave_minutes->get_value()); 
}

int EnvironmentDialog::get_undo()
{ return static_cast<int>(asave_minutes->get_value()); 
}

int EnvironmentDialog::get_tab_size()
{ return static_cast<int>(tab_size->get_value()); 
}

bool EnvironmentDialog::get_load_on_launch()
{ return load_on_launch->get_active(); 
}

bool EnvironmentDialog::get_replace_tabs()
{ return replace_tabs->get_active(); 
}

bool EnvironmentDialog::get_auto_indent()
{ return autoindent->get_active(); 
}

Glib::ustring get_color_string(Gdk::Color color)
{
  char blue[3], green[3], red[3];
  snprintf(blue, sizeof(blue), "%X", color.get_blue());
  snprintf(red, sizeof(red), "%X", color.get_red());
  snprintf(green, sizeof(green), "%X", color.get_green());
  
  Glib::ustring clr = "#" + Glib::ustring(red);
  if (color.get_red() == 0)
    clr += "0";
  clr += green;
  if (color.get_green() == 0)
    clr += "0";
  clr += blue;
  if (color.get_blue() == 0)
    clr += "0";
  
  return clr;
}

Glib::ustring EnvironmentDialog::get_error_color()
{ return get_color_string(error->get_color()); 
}

Glib::ustring EnvironmentDialog::get_warning_color()
{ return get_color_string(warning->get_color());
}
