/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "gotoline.h"

// This function creates the goto line toolbar by initializing the widgets, 
// creating the buttons and adding them to the horizontal box. The buttons are
// then connected to their signals
void OpenLDev::populate_goto_line_toolbar()
{
  gototoolbar = new GotoLineToolbar();
  
  // Connect the buttons to their signals
  gototoolbar->get_close()->signal_clicked().connect(sigc::mem_fun(*this, &OpenLDev::close_toolbars));
  gototoolbar->get_goto()->signal_clicked().connect(sigc::mem_fun(*this, &OpenLDev::jump_to_line_number));
}

// Toggle the visibility of the goto line toolbar. Also, this makes sure that if
// the goto line toolbar is visible, the find toolbar is hidden
void OpenLDev::on_edit_goto_line() 
{
  if (gototoolbar->is_visible())
    jump_to_line_number();
  else
  {
    gototoolbar->show();
    findtoolbar->hide();
  }
}

// Close both toolbars. The find toolbar and the goto line toolbar.
void OpenLDev::close_toolbars()
{
  gototoolbar->hide();
  findtoolbar->hide();
}

// Move to the specified line number in the buffer. If the line exists, the
// cursor is moved to the beginning of that line and the notebookmanager scrolls
// to it. If the line does not exist (e.g. It is out of range), then the goto
// line error message and image are shown, not to be hidden until a successful
// goto line is completed.
void OpenLDev::jump_to_line_number()
{
  if (noteman->get_file_open())
  {
    Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_textbuffer();
  
    // If the line is within the bounds of the document
    if (buffer->get_line_count() >= gototoolbar->get_goto_value())
    {
      Gtk::TextIter iter = buffer->get_iter_at_line(gototoolbar->get_goto_value()-1);
      buffer->move_mark(buffer->get_insert(), iter);
      buffer->move_mark(buffer->get_selection_bound(), iter);
      noteman->scroll_to_mark();
      set_focus(*noteman->get_textview());
    
      gototoolbar->show_error(false);
    }
    // If the line is out of bounds...
    else
      gototoolbar->show_error(true);
  }
}
