/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "template.h"

Glib::ustring get_attribute_ustring(Glib::ustring, Glib::ustring&);
void get_attribute_vector(Glib::ustring, Glib::ustring&, vector<Glib::ustring>&);

// Load the template file listed at the location received.
void ProjectTemplate::load_template (Glib::ustring file)
{
  Glib::ustring filename = string(INSTALL_PREFIX) + "/share/openldev/templates/" + file + ".template";
  Glib::ustring text = read_file(filename);
  
  tar = get_attribute_ustring("<template>", text);
  title = get_attribute_ustring("<title>", text);
  language = get_attribute_ustring("<language>", text);
  compile = get_attribute_ustring("<compile>", text);
  build = get_attribute_ustring("<build>", text);
  autogen = get_attribute_ustring("<autogen>", text);
  configure = get_attribute_ustring("<configure>", text);
  clean = get_attribute_ustring("<clean>", text);
  single = get_attribute_ustring("<comment_single>", text);
  multi_begin = get_attribute_ustring("<comment_multi_begin>", text);
  multi_end = get_attribute_ustring("<comment_multi_end>", text);
  
  get_attribute_vector("<extensions>", text, extensions);
  get_attribute_vector("<variable>", text, variables);
  get_attribute_vector("<keywords>", text, keywords);
}

// Get ustring data from the specified attribute in the text. This function
// receives the attribute to find, the text of the environment settings file
// and the value to return if the attribute does not exist.
Glib::ustring get_attribute_ustring(Glib::ustring attribute, Glib::ustring &text)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find(attribute, start+1) - start;
    return text.substr(start, len);
  }
  else
    return "";
}

// Get ustring data from the specified attribute in the text. This function
// receives the attribute to find, the text of the environment settings file
// and the value to return if the attribute does not exist.
void get_attribute_vector(Glib::ustring attribute, Glib::ustring &text, vector<Glib::ustring> &list)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find(attribute, start) - start;
    
    Glib::ustring txt = text.substr(start, len);
    
    while (!txt.empty())
    {
      len = txt.find(',', 0);
      if (len != Glib::ustring::npos)
      {
        list.push_back(txt.substr(0, len));
        txt.erase(0, len+1);
      }
      else
      {
        list.push_back(txt);
        txt.erase(0, txt.length());
      }
    }
  }
}

// Make the compile string replacing {SOURCE} with the filename of the
// file that is to be compiled
Glib::ustring ProjectTemplate::get_compile (Glib::ustring fn)
{
  int location = compile.find("{SOURCE}", 0);
  
  Glib::ustring command = compile;
  command.erase(location, 8);
  command.insert(location, fn);
  return command;
}

// Load the templates from the /usr/share/openldev/templates/ folder. The
// index.template file is no longer used, so this function will be renamed
// in the future...
void TemplateList::load_index_file()
{
  string directory = string(INSTALL_PREFIX) + "/share/openldev/templates/", name;
  int start, len;

  Glib::Dir dir(directory);
  vector<string> entries (dir.begin(), dir.end());
  
  for (unsigned int l=0; l < entries.size(); l++)
    if (entries.at(l).find(".tar", 0) != string::npos)
      file.push_back(entries.at(l).substr(0, entries.at(l).length()-4));
  
  for (unsigned int i=0; i < file.size(); i++)
  {
    Glib::ustring text = read_file(string(INSTALL_PREFIX) + "/share/openldev/templates/" + file.at(i) + ".template");
    start = text.find("<title>", 0) + 7;
    len = text.find("<title>", start) - start;
    title.push_back(text.substr(start, len));
    start = text.find("<defaultfiles>", 0) + 14;
    len = text.find("<defaultfiles>", start) - start;
    defaultfiles.push_back(text.substr(start, len));
  }
}

// List of functions to receive the values of private data
int ProjectTemplate::get_variables_size()
{ return variables.size(); 
}

int ProjectTemplate::get_keywords_size()
{ return keywords.size(); 
}

Glib::ustring ProjectTemplate::get_variable_name(int loc)
{ return variables.at(loc); 
}

Glib::ustring ProjectTemplate::get_keyword_name (int loc)
{ return keywords.at(loc); 
}

Glib::ustring ProjectTemplate::get_language()
{ return language; 
}

Glib::ustring ProjectTemplate::get_build()
{ return build; 
}

Glib::ustring ProjectTemplate::get_autogen()
{ return autogen; 
}

Glib::ustring ProjectTemplate::get_configure()
{ return configure; 
}

Glib::ustring ProjectTemplate::get_clean()
{ return clean; 
}

Glib::ustring ProjectTemplate::get_single_comment()
{ return single; 
}

Glib::ustring ProjectTemplate::get_multi_comment_begin()
{ return multi_begin; 
}

Glib::ustring ProjectTemplate::get_multi_comment_end()
{ return multi_end; 
}
